#' @keywords internal
#'
#' @section Datasets:
#' * [aids]: a doubly truncated sample of AIDS incubation times from patients
#'   with transfusion-acquired HIV.
#' * [nonprop_sample]: a simulated sample with non-proportional hazards and
#'   quasi-independent double truncation.
#'
#' @section Key functions:
#'
#' **Survival objects to hold doubly truncated data**
#' * [Survdt()]: one observation per subject.
#' * [Survdt2()]: for (start,stop] data with multiple observations per subject.
#' * [time_split()], [survdt2to1()]: convert between the two formats above.
#'
#' **Nonparametric analysis under double truncation**
#' * [npmle()]: nonparametric maximum likelihood estimation.
#'   The output has a `plot` method to plot the estimated survival function,
#'   cumulative hazard, and selection probabilities along with pointwise
#'   confidence intervals.
#'   The nonparametric maximum likelihood estimator (NPMLE) is used in the two
#'   tests that follow.
#' * [test_ignorability_indtrunc()]: test for non-ignorable sampling bias.
#' * [test_samplediff_indtrunc()]: test for any difference in event time
#'   distribution across groups.
#' * [test_quasiindep_ctau()]: test for dependence between truncation times and
#'   the event time.
#'   The NPMLE requires quasi-independence.
#'
#' **Cox regression under double truncation**
#' * [coxph_indtrunc()]: inverse probability weighted Cox regression with options
#'   for robust time-varying weights estimated nonparametrically.
#'   The output has common methods such as `confint`, `residuals`, and `vcov`.
#' * [plot_coxsurv()]: plot conditional survival functions or cumulative hazards
#'   from a fitted Cox model along with pointwise confidence intervals.
#' * [test_quasiindep_covariates()]: test for dependence between truncation times
#'   and the event time or covariates.
#'   The nonparametric inverse probability weighting used by [coxph_indtrunc()]
#'   requires quasi-independence.
#' * [positivity_sens_indtrunc()]: sensitivity analysis for positivity violations.
#'   Check how the estimates and confidence intervals may change under a range
#'   of values for the baseline probability mass potentially lost due to zero
#'   sampling probability.
#'
#' @section Vignettes:
#' If vignettes were built when installing [survdt], see
#' * `vignette("nonparametric-analysis", package = "survdt")`: an example of
#'   various nonparametric analyses applied to the [aids] data.
#' * `vignette("cox-regression-quasiindep", package = "survdt")`: an example of
#'   Cox regression and its assumption diagnostics applied to the [aids] data.
#'
#'
#' @references
#' Vazquez, O., & Xie, S. X. (2025). Robust inverse probability weighted
#' estimators for doubly truncated Cox regression with closed-form standard
#' errors. *Lifetime Data Analysis*, **31**(2), 364–-393.
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL
