% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nonprop_sample}
\alias{nonprop_sample}
\title{Simulated data with non-proportional hazards and independent double truncation}
\format{
A data frame with 490 rows and 5 columns:
\describe{
\item{\code{ltrunc}}{The left truncation time.}
\item{\code{rtrunc}}{The right truncation time.}
\item{\code{event_time}}{The event time.}
\item{\code{group}}{A binary covariate with non-proportional hazards.}
\item{\code{x}}{A continuous covariate with proportional hazards.}
}
}
\usage{
nonprop_sample
}
\description{
The event times were simulated from a Cox model with hazard function
\deqn{\lambda(t|x,\textit{group}) = \exp\{0.7\textit{group}\times 1(t \leq 0.6) - 0.5x\}\lambda_0(t-0.1)}
where \eqn{\lambda_0(t)} is the hazard function from a beta(2,2) distribution.
The covariates \eqn{\textit{group}} and \eqn{x} were independent bernoulli(0.5)
and uniform(0,1) random variates, respectively.

The left truncation time had a uniform(0,0.6) distribution,
while the time between the left and right truncation time was an independent
uniform(0.1,0.7) random variate.
All times were rounded to two decimal places.
}
\keyword{datasets}
