% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{aids}
\alias{aids}
\title{AIDS incubation data}
\format{
A data frame with 295 rows and 5 columns:
\describe{
\item{\code{incu}}{The incubation time, i.e. months from HIV infection to AIDS
diagnosis.}
\item{\code{age}}{The age at HIV infection in years.}
\item{\code{ltrunc}}{The time from HIV infection to the start of data collection
(1982), in months.
Note that \code{ltrunc} is negative for patients infected before 1982.}
\item{\code{rtrunc}}{The time from HIV infection to the end of data collection
(July 1986), in months.}
\item{\code{age_gp}}{A factor with three \code{age} groups: \verb{<=4}, \code{4-59}, and \verb{>59}
(the baseline level).}
}
}
\source{
Wang, M. -C. (1989). A semiparametric model for randomly truncated data.
\emph{Journal of the American Statistical Association}, \strong{84}(407), 742--748.
}
\usage{
aids
}
\description{
A doubly truncated dataset of AIDS incubation times from patients with
transfusion-acquired HIV collected by the Centers for Disease Control and
Prevention (CDC). The sample contains patients who were diagnosed between
the discovery of the disease (1982) and the end of study (July 1986), with
infection times determined retrospectively from a blood transfusion registry.
}
\keyword{datasets}
